/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

'use strict';

var ReactNativeBaseComponent = require('./ReactNativeBaseComponent');
var ReactNativeFeatureFlags = require('./ReactNativeFeatureFlags');
var ReactNativeViewConfigRegistry = require('./ReactNativeViewConfigRegistry');

// See also ReactNativeBaseComponent


/**
 * @param {string} config iOS View configuration.
 * @private
 */
var createReactNativeFiberComponentClass = function (viewConfig) {
  return ReactNativeViewConfigRegistry.register(viewConfig);
};

/**
 * @param {string} config iOS View configuration.
 * @private
 */
var createReactNativeComponentClass = function (viewConfig) {
  var Constructor = function (element) {
    this._currentElement = element;
    this._topLevelWrapper = null;
    this._hostParent = null;
    this._hostContainerInfo = null;
    this._rootNodeID = 0;
    this._renderedChildren = null;
  };
  Constructor.displayName = viewConfig.uiViewClassName;
  Constructor.viewConfig = viewConfig;
  Constructor.propTypes = viewConfig.propTypes;
  Constructor.prototype = new ReactNativeBaseComponent(viewConfig);
  Constructor.prototype.constructor = Constructor;

  return Constructor;
};

module.exports = ReactNativeFeatureFlags.useFiber ? createReactNativeFiberComponentClass : createReactNativeComponentClass;